class SessionValidateException(Exception):
    pass


class SessionBuildException(Exception):
    pass


class SessionExecuteException(Exception):
    pass


class SessionDisplayException(Exception):
    pass
